local m = {}


local ButtonOn = love.graphics.newImage("Assets_Images/UI/ButtonOn.png")
local ButtonOff = love.graphics.newImage("Assets_Images/UI/ButtonOff.png")

local CheckOn = love.graphics.newImage("Assets_Images/UI/CheckBoxOn.png")
local CheckOff = love.graphics.newImage("Assets_Images/UI/CheckBoxOff.png")

local SliderBar = love.graphics.newImage("Assets_Images/UI/SliderBar.png")
local SliderBarVert = love.graphics.newImage("Assets_Images/UI/SliderBarVert.png")
local SliderKnob = love.graphics.newImage("Assets_Images/UI/SliderKnob.png")

local FrameBody = love.graphics.newImage("Assets_Images/UI/Frame.png")

-- define the fonts
local Font = {
	Normal = love.graphics.newFont(25)}--[[,
	Bold = love.graphics.newFont("Assets_Scripts/TimesBold.ttf",25),
	Italic = love.graphics.newFont("Assets_Scripts/TimesItalic.ttf",25),
	BoldItalic = love.graphics.newFont("Assets_Scripts/TimesBoldItalic.ttf",25),
	Stencil = love.graphics.newFont("Assets_Scripts/Stencil.ttf",25),
	Drawn = love.graphics.newFont("Assets_Scripts/MonotypeCorsiva.ttf",25)
}--]]

function MouseInRect(LeftX,UpperY,RightX,LowerY)
	local MouseX,MouseY = love.mouse.getX(),love.mouse.getY()
	local LX = MouseX>LeftX
	local RX = MouseX<RightX
	local UY = MouseY>UpperY
	local LY = MouseY<LowerY
	if LX and RX and UY and LY then
		return true
	end
	return false
end

function Lerp(A,B,T)
	return ((B-A)*T)+A
end

function Clamp(A,B,V)
	return math.max(A,math.min(B,V))
end

function m.UIElement()
	local Element = {}
	Element.Pos = {0,0,0,0}
	Element.Size = {0,0,0,0}
	Element.AbsolutePos = {0,0}
	Element.AbsoluteSize = {0,0}
	Element.Children = {}
	Element.ChildOffset = {0,0,0,0}
	Element.Active = false
	Element.OnMouseDown = function()end
	Element.OnMouseUp = function()end
	Element.Draw = function(MouseX,MouseY)
		love.graphics.draw(FrameBody,Element.AbsolutePos[1],Element.AbsolutePos[2],0,Element.AbsoluteSize[1]/512,Element.AbsoluteSize[2]/512)
    end
    Element.NewChild = function(self,ChildId,ChildType)
		print(self,ChildId,ChildType)
		self.Children[ChildId] = ChildType()
		self.Children[ChildId].Parent = self
		return self.Children[ChildId]
	end
    return Element
end

function m.Group()
	local Element = m.UIElement()
	Element.Draw = function()end
	return Element
end

function m.Image()
	local Element = m.UIElement()
	Element.Image = nil
	Element.Scale = {1,1}
	Element.Draw = function()
		love.graphics.draw(Element.Image,Element.AbsolutePos[1],Element.AbsolutePos[2],0,Element.Scale[1],Element.Scale[2])
	end
	return Element
end

function m.Button()
	local Element = m.UIElement()
	Element.ClickSound = true
	Element.Text = "Button"
	Element.TextAlign = 1
	Element.FontStyle = "Normal"
	Element.Draw = function(MouseX,MouseY)
		if MouseInRect(Element.AbsolutePos[1],Element.AbsolutePos[2],Element.AbsolutePos[1]+Element.AbsoluteSize[1],Element.AbsolutePos[2]+Element.AbsoluteSize[2]) then
			love.graphics.draw(ButtonOn,Element.AbsolutePos[1],Element.AbsolutePos[2],0,Element.AbsoluteSize[1]/256,Element.AbsoluteSize[2]/256)
		else
			love.graphics.draw(ButtonOff,Element.AbsolutePos[1],Element.AbsolutePos[2],0,Element.AbsoluteSize[1]/256,Element.AbsoluteSize[2]/256)
		end
		local TextX = Element.TextAlign*((Element.AbsoluteSize[1]-Font[Element.FontStyle]:getWidth(Element.Text))/2)
		love.graphics.setFont(Font[Element.FontStyle])
		love.graphics.setColor(0,0,0)
		love.graphics.print(Element.Text,Element.AbsolutePos[1]+TextX+1,Element.AbsolutePos[2]+6)
		love.graphics.setColor(0.95,0.95,0.6)
		love.graphics.print(Element.Text,Element.AbsolutePos[1]+TextX,Element.AbsolutePos[2]+5)
	end
	return Element
end

function m.Label()
	local Element = m.UIElement()
	Element.Text = "Label"
	Element.TextAlign = 0
	Element.FontStyle = "Normal"
	Element.Draw = function()
		local TextX = Element.TextAlign*((Element.AbsoluteSize[1]-Font[Element.FontStyle]:getWidth(Element.Text))/2)
		love.graphics.setFont(Font[Element.FontStyle])
		love.graphics.setColor(0,0,0)
		love.graphics.print(Element.Text,Element.AbsolutePos[1]+TextX+1,Element.AbsolutePos[2]+6)
		love.graphics.setColor(0.95,0.95,0.6)
		love.graphics.print(Element.Text,Element.AbsolutePos[1]+TextX,Element.AbsolutePos[2]+5)
	end
	return Element
end

function m.Slider()
	local Element = m.UIElement()
	Element.Min = 0
	Element.Max = 100
	Element.Value = 50
	Element.Integer = true
	Element.ShowValue = true
	Element.FontStyle = "Normal"
	Element.OnMouseUp = function()Element.Active = false end
	Element.OnUpdated = function()end
	Element.Draw = function(MouseX)
	
		-- draw the bar
		
		love.graphics.draw(SliderBar,Element.AbsolutePos[1],Element.AbsolutePos[2],0,Element.AbsoluteSize[1]/256,25/256)
		local KnobX = Element.AbsolutePos[1]
		local KnobXMax = (Element.AbsolutePos[1]+Element.AbsoluteSize[1])-256
		local KnobPercent = (Element.Value-Element.Min) / (Element.Max-Element.Min)
		love.graphics.draw(SliderKnob,Lerp(Element.AbsolutePos[1]-12.5,(Element.AbsolutePos[1]+Element.AbsoluteSize[1])-12.5,KnobPercent),Element.AbsolutePos[2],0,25/256,25/256)
		
		-- if mouse is down, update knob position and value
		
		if Element.Active then
			local ClampedMouseX = Clamp(
				Element.AbsolutePos[1],
				Element.AbsolutePos[1]+Element.AbsoluteSize[1],
				MouseX
			)
			local SliderX = (ClampedMouseX-Element.AbsolutePos[1]) / Element.AbsoluteSize[1]
			if Element.Integer then
				Element.Value = math.floor(Lerp(Element.Min,Element.Max,SliderX))
			else
				Element.Value = Lerp(Element.Min,Element.Max,SliderX)
			end
			
			Element.OnUpdated()
			
			if Element.ShowValue then
				love.graphics.setFont(_G.Font[Element.FontStyle])
				love.graphics.setColor(0,0,0)
				love.graphics.print(math.floor(Element.Value*100)*0.01,ClampedMouseX+11,Element.AbsolutePos[2]+4)
				love.graphics.setColor(0.95,0.95,0.6)
				love.graphics.print(math.floor(Element.Value*100)*0.01,ClampedMouseX+10,Element.AbsolutePos[2]+3)
			end
			if (not MouseInRect(Element.AbsolutePos[1],Element.AbsolutePos[2],Element.AbsolutePos[1]+Element.AbsoluteSize[1],Element.AbsolutePos[2]+Element.AbsoluteSize[2])) and (not love.mouse.isDown(1)) then
				Element.OnMouseUp()
			end
		end
	end
	return Element
end

function m.VerticalSlider()
	local Element = m.UIElement()
	Element.Min = 0
	Element.Max = 100
	Element.Value = 50
	Element.Integer = true
	Element.ShowValue = true
	Element.FontStyle = "Normal"
	Element.OnMouseUp = function()Element.Active = false end
	Element.OnUpdated = function()end
	Element.Draw = function(MouseX,MouseY)
	
		-- draw the bar
		
		love.graphics.draw(SliderBarVert,Element.AbsolutePos[1],Element.AbsolutePos[2],0,25/256,Element.AbsoluteSize[2]/256)
		local KnobY = Element.AbsolutePos[2]
		local KnobYMax = (Element.AbsolutePos[2]+Element.AbsoluteSize[2])-256
		local KnobPercent = (Element.Value-Element.Min) / (Element.Max-Element.Min)
		love.graphics.draw(SliderKnob,Element.AbsolutePos[1],Lerp(Element.AbsolutePos[2]-12.5,(Element.AbsolutePos[2]+Element.AbsoluteSize[2])-12.5,KnobPercent),0,25/256,25/256)
		
		-- if mouse is down, update knob position and value
		
		if Element.Active then
			local ClampedMouseY = Clamp(
				Element.AbsolutePos[2],
				Element.AbsolutePos[2]+Element.AbsoluteSize[2],
				MouseY
			)
			local SliderY = (ClampedMouseY-Element.AbsolutePos[2]) / Element.AbsoluteSize[2]
			if Element.Integer then
				Element.Value = math.floor(Lerp(Element.Min,Element.Max,SliderY))
			else
				Element.Value = Lerp(Element.Min,Element.Max,SliderY)
			end
			
			Element.OnUpdated()
			
			if Element.ShowValue then
				love.graphics.setFont(_G.Font[Element.FontStyle])
				love.graphics.setColor(0,0,0)
				love.graphics.print(math.floor(Element.Value*100)*0.01,Element.AbsolutePos[1]+4,ClampedMouseY+11)
				love.graphics.setColor(0.95,0.95,0.6)
				love.graphics.print(math.floor(Element.Value*100)*0.01,Element.AbsolutePos[1]+3,ClampedMouseY+10)
			end
			if (not MouseInRect(Element.AbsolutePos[1],Element.AbsolutePos[2],Element.AbsolutePos[1]+Element.AbsoluteSize[1],Element.AbsolutePos[2]+Element.AbsoluteSize[2])) and (not love.mouse.isDown(1)) then
				Element.OnMouseUp()
			end
		end
	end
	return Element
end

function m.CheckBox()
	local Element = m.UIElement()
	Element.Text = "Check Box"
	Element.Value = false
	Element.FontStyle = "Normal"
	Element.ClickSound = true
	Element.OnUpdated = function()end
	Element.OnMouseDown = function()
		Element.Value = not Element.Value
		Element.OnUpdated()
	end
	Element.Draw = function()
		if Element.Value then
			love.graphics.draw(CheckOn,Element.AbsolutePos[1],Element.AbsolutePos[2],0,50/256,50/256)
		else
			love.graphics.draw(CheckOff,Element.AbsolutePos[1],Element.AbsolutePos[2],0,50/256,50/256)
		end
		
		love.graphics.setFont(_G.Font[Element.FontStyle])
		love.graphics.setColor(0,0,0)
		love.graphics.print(Element.Text,Element.AbsolutePos[1]+56,Element.AbsolutePos[2]+11)
		love.graphics.setColor(0.95,0.95,0.6)
		love.graphics.print(Element.Text,Element.AbsolutePos[1]+55,Element.AbsolutePos[2]+10)
	end
	return Element
end

return m
