--[[
	== CONTROLS ==

	Arrow Keys - Camera Movement
	WASD - Camera Movement
	F - Fill
	C - Copy
	V - Paste
	Space - Swap between editing screen and tile selection screen
	J - Previous Layer
	K - Next Layer
	Holding all arrow keys then pressing enter on the tile selection screen - The blow everybody up function (Will wipe your progress)
	P - Save
	[ - Generate Preview (BROKEN, WILL CRASH GAME)
	Plus - Zoom In
	Dash - Zoom Out
	0 - Default Zoom
--]]

love.graphics.setDefaultFilter("nearest","nearest")
-- Source Modules
--local UI = require("Assets_Scripts/Gspot"):setComponentMax("native")
--local UIElements = require("Assets_Scripts/UIElements")
local TilesetLoader = require("Assets_Scripts/TilesetLoader")
local Tileset = TilesetLoader.LoadTileset("Assets_Images/Tiles.png",32,32,24,24)
local ChunkCompression = require("Assets_Scripts/ChunkCompression")

-- Game Directory Modules
local Animated = require("Assets_Scripts/AnimatedDrawables")
local Tiles = require("Assets_Scripts/Tiles")
local ChunkData = {
	Music = "Overworld",
	BgTop = "#CEFEFF",
	BgBottom = "#3AB0FF",
	BgExtra = "None",
	Loaded = "true"
}

local EraserGraphic = love.graphics.newImage("Assets_Images/Eraser.png")
local CurrentTile = "Block"
local ScrollbarPos = 0
local DarkBg = false

local CamX = 0
local CamY = 0
local CamZoom = 1
local ZoomSpeed = 0.01
local CamSpeed = 5
local Preview

local Clipboard = {{"Block"}}
local Copying = false
local CopyCoord1 = {0,0}
local Pasting = false
local MouseDebounce = false

local MouseTileX = 0
local MouseTileY = 0
local MouseOnBoard = false

local CurrentLayer = 1

local MouseTileSelection = 0
local Filling = false
local FillCoord1 = {0,0}

local Screen = 0
local TilesListing = {}
for k,v in pairs(Tiles.Tiles) do
	table.insert(TilesListing,{k,v})
end
table.sort(TilesListing,function(a,b) return a[1] < b[1] end)

love.graphics.setFont(love.graphics.newFont(25))

for X=1,32,1 do
	ChunkData[X] = {}
	for Y=1,32,1 do
		ChunkData[X][Y] = {}
		for Z=0,2,1 do
			ChunkData[X][Y][Z] = "~"
		end
	end
end

local Right = love.graphics.getWidth()
local Bottom = love.graphics.getHeight()
function love.update(Delta)
	local DeltaTime = 1-Delta
	local MouseX = love.mouse.getX()
	local MouseY = love.mouse.getY()
	Right,Bottom = love.graphics.getDimensions()
	
	if love.keyboard.isDown("up") or love.keyboard.isDown("w") then
		CamY = CamY + CamSpeed*DeltaTime
	end
	if love.keyboard.isDown("down") or love.keyboard.isDown("s") then
		CamY = CamY - CamSpeed*DeltaTime
	end
	if love.keyboard.isDown("left") or love.keyboard.isDown("a") then
		CamX = CamX + CamSpeed*DeltaTime
	end
	if love.keyboard.isDown("right") or love.keyboard.isDown("d") then
		CamX = CamX - CamSpeed*DeltaTime
	end
	if love.keyboard.isDown("=") then
		CamZoom = math.min(2,CamZoom + ZoomSpeed)
	end
	if love.keyboard.isDown("-") then
		CamZoom = math.max(0.25,CamZoom - ZoomSpeed)
	end
	MouseTileX = math.floor((MouseX/CamZoom - CamX) / 32)
	MouseTileY = math.floor((MouseY/CamZoom - CamY) / 32)
	MouseOnBoard = (Screen == 0) and
		(MouseTileX > 0) and
		(MouseTileX < 33) and
		(MouseTileY > 0) and
		(MouseTileY < 33)
	if Screen == 0 then
		if MouseOnBoard and love.mouse.isDown(1) then
			if Filling then
				Fill(FillCoord1,{MouseTileX,MouseTileY})
				Filling = false
			end
			if Copying then
				Copy(CopyCoord1,{MouseTileX,MouseTileY})
				Copying = false
			end
			if Pasting then
				Paste()
				Pasting = false
			end
			if not MouseDebounce then
				ChunkData[MouseTileX][MouseTileY][CurrentLayer] = CurrentTile
			end
		end
	end
	if Screen == 1 then
		if love.mouse.isDown(1) and MouseX > Right-25 then
			ScrollbarPos = MouseY/Bottom
		end
		MouseTileSelection = math.floor((MouseY+((#TilesListing*64)*ScrollbarPos))/64) + 1
	end
	Animated.UpdateTick()
end

function Fill(A,B)
	local XDir = 1
	local YDir = 1
	if A[1] > B[1] then XDir = -1 end
	if A[2] > B[2] then YDir = -1 end
	for X=A[1],B[1],XDir do
		if not ChunkData[X] then
			break
		end
		for Y=A[2],B[2],YDir do
			if not ChunkData[X][Y] then
				break
			end
			ChunkData[X][Y][CurrentLayer] = CurrentTile
		end
	end
	MouseDebounce = true
end

function RenderTile(TileType,X,Y)
	if TileType ~= "~" then
		local Tile = Tiles.Tiles[TileType]
		if not Tile then return end
		if Tile.Animated ~= "" then
			love.graphics.draw(Animated.GetDrawable(Tile.Animated),X*CamZoom,Y*CamZoom,0,CamZoom,CamZoom)
		else
			love.graphics.draw(Tileset[Tile.TileSetX][Tile.TileSetY],X*CamZoom,Y*CamZoom,0,CamZoom,CamZoom)
		end
	end
end

function Lerp(A,B,T)
	local V0 = math.min(A,B)
	local V1 = math.max(A,B)
	return ((V1-V0)*T)+V0
end

function love.draw()
	local MouseX = love.mouse.getX()
	local MouseY = love.mouse.getY()
	Right,Bottom = love.graphics.getDimensions()
	if Screen == 0 then
		local BgTop = {
			r=tonumber("0x"..string.sub(ChunkData.BgTop,2,3)),
			g=tonumber("0x"..string.sub(ChunkData.BgTop,4,5)),
			b=tonumber("0x"..string.sub(ChunkData.BgTop,6,7))
		}
		local BgBottom = {
			r=tonumber("0x"..string.sub(ChunkData.BgBottom,2,3)),
			g=tonumber("0x"..string.sub(ChunkData.BgBottom,4,5)),
			b=tonumber("0x"..string.sub(ChunkData.BgBottom,6,7))
		}
		if DarkBg then
			BgTop.r=BgTop.r*0.5
			BgTop.g=BgTop.g*0.5
			BgTop.b=BgTop.b*0.5
			BgBottom.r=BgBottom.r*0.5
			BgBottom.g=BgBottom.g*0.5
			BgBottom.b=BgBottom.b*0.5
		end
		for Y=1,1024,1 do
			love.graphics.setColor(
				Lerp(BgBottom.r,BgTop.r,Y/1024)/255,
				Lerp(BgBottom.g,BgTop.g,Y/1024)/255,
				Lerp(BgBottom.b,BgTop.b,Y/1024)/255
			)
			love.graphics.rectangle("fill",(CamX+32)*CamZoom,(CamY+Y+32)*CamZoom,1024*CamZoom,CamZoom)
		end
		love.graphics.setColor(1,1,1)
		love.graphics.setLineWidth(2)
		for X=1,32,1 do
			for Y=1,32,1 do
				love.graphics.setColor(1,1,1,0.25)
				if (MouseTileX == X) and (MouseTileY == Y) then
					if CurrentTile == "~" then love.graphics.setColor(1,0,0,0.25) end
					love.graphics.rectangle("fill",(CamX+(X*32))*CamZoom,(CamY+(Y*32))*CamZoom,32*CamZoom,32*CamZoom)
				elseif ChunkData[X][Y][CurrentLayer] == "~" then
					love.graphics.rectangle("line",(CamX+(X*32))*CamZoom,(CamY+(Y*32))*CamZoom,32*CamZoom,32*CamZoom)
				end
				love.graphics.setColor(1,1,1)
				RenderTile(ChunkData[X][Y][CurrentLayer],CamX+(X*32),CamY+(Y*32))
			end
		end
		if Filling then
			love.graphics.setColor(0,0,1)
			love.graphics.rectangle("line",
				(CamX+math.min(FillCoord1[1],MouseTileX)*32)*CamZoom,
				(CamY+math.min(FillCoord1[2],MouseTileY)*32)*CamZoom,
				32*(math.abs(FillCoord1[1]-MouseTileX)+1)*CamZoom,
				32*(math.abs(FillCoord1[2]-MouseTileY)+1)*CamZoom
			)
		end
		if Copying then
			love.graphics.setColor(1,1,0)
			love.graphics.rectangle("line",
				(CamX+math.min(CopyCoord1[1],MouseTileX)*32)*CamZoom,
				(CamY+math.min(CopyCoord1[2],MouseTileY)*32)*CamZoom,
				32*(math.abs(CopyCoord1[1]-MouseTileX)+1)*CamZoom,
				32*(math.abs(CopyCoord1[2]-MouseTileY)+1)*CamZoom
			)
		end
		if Pasting then
			local StartX = MouseTileX-math.floor(#Clipboard/2)
			local StartY = MouseTileY-math.floor(#Clipboard[1]/2)
			love.graphics.setColor(0,1,0)
			love.graphics.rectangle("line",
				(CamX+math.min(StartX,StartX+#Clipboard)*32)*CamZoom,
				(CamY+math.min(StartY,StartY+#Clipboard[1])*32)*CamZoom,
				32*#Clipboard*CamZoom,
				32*#Clipboard[1]*CamZoom
			)
		end
		love.graphics.setColor(1,1,1,0.5)
		RenderTile(CurrentTile,(MouseX-64)/CamZoom,(MouseY-64)/CamZoom)
		love.graphics.setColor(1,1,1)
		love.graphics.print({{1,1,1},"(",{1,0,0},MouseTileX,{1,1,1},", ",{0,1,0},MouseTileY,{1,1,1},", ",{0,0,1},CurrentLayer,{1,1,1},")"},5,Bottom-30,0,1,1)
	elseif Screen == 1 then
		love.graphics.setColor(0.5,0.5,0.5)
		love.graphics.rectangle("fill",Right-25,0,25,Bottom)
		love.graphics.setColor(0.75,0.75,0.75)
		love.graphics.rectangle("fill",Right-25,(Bottom*ScrollbarPos)-12.5,25,25)
		love.graphics.setColor(1,1,1)
		love.graphics.draw(EraserGraphic,0,-((#TilesListing*64)*ScrollbarPos),0,2,2)
		love.graphics.print("Air",64,-((#TilesListing*64)*ScrollbarPos),0,1,1)
		if MouseTileSelection-1 == 0 then
			love.graphics.rectangle("line",0,-((#TilesListing*64)*ScrollbarPos),Right-25,64)
		end
		for k,v in pairs(TilesListing) do
			local Y = (64*k)-((#TilesListing*64)*ScrollbarPos)
			if TilesListing[MouseTileSelection-1] then
				if MouseTileSelection-1 > 0 then
					if TilesListing[MouseTileSelection-1][1] == v[1] then
						love.graphics.rectangle("line",0,Y,Right-25,64)
					end
				end
			end
			if v[2].Animated ~= "" then
				love.graphics.draw(Animated.GetDrawable(v[2].Animated),0,Y,0,2,2)
			else
				love.graphics.draw(Tileset[v[2].TileSetX][v[2].TileSetY],0,Y,0,2,2)
			end
			love.graphics.print(v[1],64,Y,0,1,1)
		end
	elseif Screen == 2 then
		if Preview then
			local SmallestDimension = math.min(love.graphics.getDimensions())
			local Scale = SmallestDimension/1024
			love.graphics.draw(Preview,0,0,0,Scale,Scale)
		end
	end
end

function love.filedropped(FileDropped)
	local Content = ""
	for Line in FileDropped:lines() do
		Content = Content .. Line .. "\n"
	end
	local NewChunkData = ChunkCompression.DecompressChunk(Content)
	for X=1,32,1 do
		for Y=1,32,1 do
			for Z=0,2,1 do
				ChunkData[X][Y][Z] = NewChunkData[X][Y][Z]
			end
		end
	end
	for k,v in pairs(NewChunkData) do
		if not tonumber(k) then
			ChunkData[k] = v
		end
	end
end
--[[
function love.directorydropped(Path)
	local CorrectPath = string.gsub(Path,"\\","/")
	--love.filesystem.setRequirePath(Path.."/?.lua;"..Path.."\\?.lua")
	Animated = require(CorrectPath.."/Assets_Scripts/AnimatedDrawables.lua")
	Tileset = TilesetLoader.LoadTileset(CorrectPath.."/Assets_Images/Tileset.png")
	Tiles = require(CorrectPath.."/Assets_Scripts/Tiles")
end
--]]

function love.mousereleased(MouseX,MouseY,Button)
	if Screen == 1 then
		if MouseX < Right-25 and MouseTileSelection > -1 and MouseTileSelection-1 <= #TilesListing then
			if MouseTileSelection-1 == 0 then
				CurrentTile = "~"
			else
				CurrentTile = TilesListing[MouseTileSelection-1][1]
			end
			Screen = 0
		end
	end
	MouseDebounce = false
end

function Paste()
	local StartX = MouseTileX-math.floor(#Clipboard/2)-1
	local StartY = MouseTileY-math.floor(#Clipboard[1]/2)-1
	for X=1,#Clipboard,1 do
		if not ChunkData[X+StartX] then
			break
		end
		for Y=1,#Clipboard[1],1 do
			if not ChunkData[X+StartX][Y+StartY] then
				break
			end
			ChunkData[X+StartX][Y+StartY][CurrentLayer] = Clipboard[X][Y]
		end
	end
	MouseDebounce = true
end

function TheBlowEverybodyUpFunction()
	for X=1,32,1 do
		for Y=1,32,1 do
			for Z=0,2,1 do
				ChunkData[X][Y][Z] = TilesListing[math.random(1,#TilesListing)][1]
			end
		end
	end
end

function Copy(A,B)
	Clipboard = {}
	local XDir = 1
	local YDir = 1
	if A[1] > B[1] then XDir = -1 end
	if A[2] > B[2] then YDir = -1 end
	for X=A[1],B[1],XDir do
		--[[
		if not ChunkData[X] then
			Clipboard = {{"Block"}}
			break
		end--]]
		table.insert(Clipboard,{})
		for Y=A[2],B[2],YDir do
			if not ChunkData[X] then
				table.insert(Clipboard[#Clipboard],"~")
			else
				if not ChunkData[X][Y] then
					table.insert(Clipboard[#Clipboard],"~")
				else
					table.insert(Clipboard[#Clipboard],ChunkData[X][Y][CurrentLayer])
				end
			end
		end
	end
	MouseDebounce = true
end

function love.keypressed(Key)
	if Key == "space" then
		if Screen == 0 then
			Screen = 1
		else
			Screen = 0
		end
	end
	if Screen == 0 then
		if Key == "[" then
			Screen = 2
			love.graphics.print("Generating Preview...",10,10,0,1.5,1.5)
			love.graphics.present()
			Preview = GeneratePreview()
		end
		if Key == "p" then
			love.filesystem.write("Save.chunk",ChunkCompression.CompressChunk(ChunkData))
		end
		if Key == "r" then
			CurrentTile = "~"
		end
		if Key == "f" then
			Copying = false
			Pasting = false
			if not Filling then
				Filling = true
				FillCoord1 = {MouseTileX,MouseTileY}
			else
				Fill(FillCoord1,{MouseTileX,MouseTileY})
				Filling = false
			end
		end
		if Key == "c" then
			Pasting = false
			Filling = false
			if not Copying then
				Copying = true
				CopyCoord1 = {MouseTileX,MouseTileY}
			else
				Copy(CopyCoord1,{MouseTileX,MouseTileY})
				Copying = false
			end
		end
		if Key == "v" then
			Copying = false
			Filling = false
			if not Pasting then
				Pasting = true
			else
				Paste()
				Pasting = false
			end
		end
		if Key == "j" then
			CurrentLayer = math.max(0,CurrentLayer-1)
		elseif Key == "k" then
			CurrentLayer = math.min(2,CurrentLayer+1)
		end
		if Key == "0" then
			CamZoom = 1
		end
	end
	if Screen == 1 then
		if #Key == 1 then--and string.find(Key,"%k") then
			local Char = string.lower(string.sub(Key,1,1))
			for k,v in pairs(TilesListing) do
				if string.lower(string.sub(v[1],1,1)) == Char then
					ScrollbarPos = k/#TilesListing
					break
				end
			end
		end
		if Key == "return" and love.keyboard.isDown("up") and love.keyboard.isDown("down") and love.keyboard.isDown("left") and love.keyboard.isDown("right") then
			TheBlowEverybodyUpFunction()
		end
	end
	if Screen == 2 then
		if Key == "[" then
			Screen = 0
			Preview = nil
		end
	end
end

function GeneratePreview()
	local Final = love.image.newImageData(32*32,32*32)
	for X=1,32,1 do
		for Y=1,32,1 do
			for Z=0,2,1 do
				if ChunkData[X][Y][Z] ~= "~" then
					local Tile = Tiles.Tiles[ChunkData[X][Y][Z]]
					local TileImage
					--if Tiles.Tiles[ChunkData[X][Y][Z].Animated] ~= "" then
					--	TileImage = love.image.newImageData(Animated.Animations[Tiles.Tiles[ChunkData[X][Y][Z].Animated]].Frames[1])
					--else
						TileImage = love.image.newImageData(Tileset[Tile.TileSetX][Tile.TileSetY])
					--end
					Final:paste(TileImage,0,0,(X-1)*32,(Y-1)*32,32,32)
				end
			end
		end
	end
	Final = love.graphics.newImage(Final)
	return Final
end

function love.wheelmoved(MoveX,MoveY)
	if Screen == 1 then
		local Offset = 64/(64*#TilesListing)
		ScrollbarPos = math.min(1,math.max(0,ScrollbarPos - Offset*MoveY))
	end
end

love.window.setMode(800,600,{resizable=true})
love.window.setTitle("Chunk Editor")
