love.graphics.setDefaultFilter("nearest","nearest")
love.filesystem.setIdentity(love.filesystem.getIdentity(),true)

local TilesetLoader = require("Assets_Scripts/TilesetLoader")
local Tileset
local ChunkCompression = require("Assets_Scripts/ChunkCompression")
local Animated = require("Assets_Scripts/AnimatedDrawables")
local Tiles = require("Assets_Scripts/Tiles")
local Radio = require("Assets_Scripts/Radio")
local AABB = require("Assets_Scripts/AABB")
local WorldSpace = require("Assets_Scripts/WorldSpace")
local PlaceholderChunk

local Chunks = {}

local Player = {
	X = 0,
	Y = 0,
	W = 0.8,
	H = 1.9,
	VX = 0,
	VY = 0,
	Grounded = false,
	Powerup = 0,
	State = 0,
	WalkSpeed = 0.075,
	Direction = 1,
	WallJumpAllowed = false,
	WallJumpDir = 1,
	Lives = 3
}
function HurtPlayer() if Player.Powerup > 1 then Player.Powerup = 1 elseif Player.Powerup == 1 then Player.Powerup = 0 else end end
local PlayerChunkX,PlayerChunkY = 0,0
local PlayerTileX,PlayerTileY = 0,0

local Entities = {}

local CamX = 0
local CamY = 0

local BgTop = {1,1,1}
local BgBottom = {0,0,0}

local GameScreen = 0
local GameActive = true
local SaveSlot = 1

local WindowWidth = 2
local WindowHeight = 2

function Load()
	WindowWidth = love.graphics.getWidth()
	WindowHeight = love.graphics.getHeight()
	PlaceholderChunk = ChunkCompression.DecompressChunk(require("DefaultChunk"))
	
	MarioTemplateRight = "Assets_Images/Tiles.png"
	--local MarioTemplateLeft = love.image.newImageData("Assets_Images/Mario.bmp")
	MarioSprites = {
		TilesetLoader.LoadTileset(MarioTemplateRight,32,64,14,5),
		--TilesetLoader.LoadTileset(MarioTemplateLeft,32,64,14,5)
	}
	Tileset = TilesetLoader.LoadTileset("Assets_Images/Tiles.png",32,32,24,24)
	for X=-128,128,1 do
		Chunks[X] = {}
		for Y=-128,128,1 do
			Chunks[X][Y] = PlaceholderChunk
			Chunks[X][Y].Loaded = "false"
		end
	end
end

if not love.filesystem.getInfo("Saves") then
	love.filesystem.createDirectory("Saves")
end
--[[
local ChunkLoadingThreads = {}
for X=-1,1,1 do
	for Y=-1,1,1 do
		ChunkLoadingThreads[X.."_"..Y] = love.thread.newThread(
			local LoadFunction, X, Y = ...
		)
	end
end
--]]
function RenderTile(X,Y,TileData)
	if TileData.Animated ~= "" then
		love.graphics.draw(
			Animated.GetDrawable(TileData.Animated),
			X,
			Y,
			0,
			1,
			1
		)
	else
		love.graphics.draw(
			Tileset[TileData.TileSetX][TileData.TileSetY],
			X,
			Y,
			0,
			1,
			1
		)
	end
end

function RenderChunkLayer(RenderX,RenderY,Chunk,Layer)
	for X=1,32,1 do
		for Y=1,32,1 do
			local Tile = Chunk[X][Y][Layer]
			if Tile ~= "~" then
				RenderTile((X-1)*32+RenderX,(Y-1)*32+RenderY,Tiles.Tiles[Tile])
			end
		end
	end
end

function DecodeHex(Hex)
	local Final = {}
	Final[1] = tonumber("0x"..string.sub(Hex,2,3))/255
	Final[2] = tonumber("0x"..string.sub(Hex,4,5))/255
	Final[3] = tonumber("0x"..string.sub(Hex,6,7))/255
	return Final
end

function Lerp(V0,V1,T)
	return ((V1-V0)*T)+V0
end

function RenderBg()
	local Width = WindowWidth
	local Max = love.graphics.getHeight()
	for Y=0,Max,1 do
		local R = Lerp(BgTop[1],BgBottom[1],Y/Max)
		local G = Lerp(BgTop[2],BgBottom[2],Y/Max)
		local B = Lerp(BgTop[3],BgBottom[3],Y/Max)
		love.graphics.setColor(R,G,B)
		love.graphics.rectangle("fill",0,Y,Width,1)
	end
end
---[[
function PhysicsTick()
	Player.WallJumpDir = 0

	--== PHYSICS AND CRAP SECTION ==--
	
	local OldPlayerPosX,OldPlayerPosY = Player.X,Player.Y
	-- Apply velocity
	Player.X = Player.X + Player.VX
	Player.Y = Player.Y + Player.VY
	
	--Apply gravity and friction
	Player.VX = Player.VX * 0.75
	Player.VY = math.min(0.75,Player.VY+0.05)
	
	-- Here comes the spooky part...
	-- COLLISIONS! (que inception sfx)

	-- first things first, we need to iterate through all surrounding tiles
	local FlooredPlayerX = math.floor(Player.X)
	local FlooredPlayerY = math.floor(Player.Y)
	local ShouldGround = false
	for X=FlooredPlayerX,FlooredPlayerX+2,1 do
		for Y=FlooredPlayerY,FlooredPlayerY+3,1 do
			-- Some crap used to get the tile type and class information at the current position
			local TileX,TileY,ChunkX,ChunkY = WorldSpace.GetWorldTile(X,Y)
			local TileType = Chunks[ChunkX][ChunkY][TileX+1][TileY+1][1]
			-- Only do the next crap if the tile type is not air, or else the game crashes oof
			if TileType ~= "~" then
				local Tile = Tiles.Tiles[TileType]
				
				-- Set bounding box data for tile
				-- We dont do this for the player because the X,Y,W,H can be used instead, so creating bounding box info for the player is a redundancy
				local ObstacleX = X
				local ObstacleY = Y
				local ObstacleW = 1
				local ObstacleH = 1
				-- I do this ONLY for the player's Y & H because the powerup state can affect the true height if that makes any sense at all
				local PlayerH = Player.H
				if Player.Powerup == 0 then
					PlayerH = PlayerH - 1
				end
				local PlayerY = Player.Y
				if Player.Powerup == 0 then
					PlayerY = PlayerY - 1
				end
				
				-- Modify collision box to conform with quarter blocks, only like 12 tiles actaully use this
				
				if Tile.Collision > 4 then
					ObstacleW = 0.5
					ObstacleH = 0.5
					if Tile.Collision == 6 or Tile.Collision == 8 then
						ObstacleX = ObstacleX + 0.5
					end
					if Tile.Collision > 6 then
						ObstacleY = ObstacleY + 0.5
					end
				end
				print(ObstacleW,ObstacleX)
				
				local DoCollision = true
				if Tile.Collision == 0 then
					DoCollision = false
				end
				--[[
				if DoCollision and Tile.Collision == 2 then
					if Player.VY < 0 then
						DoCollision = false
					end
				end--]]
				-- let Collision.lua handle collision detection
				if DoCollision then
					local Collision, CollisionData = AABB.CheckCollision(
						Player.X,
						Player.Y,
						Player.W,
						Player.H,
						ObstacleX,
						ObstacleY,
						ObstacleW,
						ObstacleH,
						OldPlayerPosX,
						OldPlayerPosY
					)
					if Collision then
						if CollisionData.Top and Tile.Collision ~= 2 then
							Player.Y = ObstacleY+ObstacleH
							Player.VY = 0
						end
						if CollisionData.Bottom then
							Player.Y = ObstacleY-Player.H
							Player.VY = 0
							ShouldGround = true
						end
						if CollisionData.Left and Tile.Collision ~= 2 then
							Player.X = ObstacleX+ObstacleW
							Player.VX = 0
							if Player.WallJumpAllowed then
								Player.WallJumpDir = 1
								Player.VY = Player.VY * 0.95
							end
						end
						if CollisionData.Right and Tile.Collision ~= 2 then
							Player.X = ObstacleX-Player.W
							Player.VX = 0
							if Player.WallJumpAllowed then
								Player.WallJumpDir = 2
								Player.VY = Player.VY * 0.95
							end
						end
					end
					--print(CollisionData.Top,CollisionData.Bottom,CollisionData.Left,CollisionData.Right)
				end
			end
		end
	end
	Player.Grounded = ShouldGround
end
--]]

function love.update(Delta)
	PlayerTileX,PlayerTileY,PlayerChunkX,PlayerChunkY = WorldSpace.GetWorldTile(Player.X,Player.Y)
	--if PlayerChunk.Loaded ~= "true" then
	for X=-1,1,1 do
		for Y=-1,1,1 do
			LoadChunk(PlayerChunkX+X,PlayerChunkY+Y)
		end
	end
	--end
	local PlayerChunk = Chunks[PlayerChunkX][PlayerChunkY]
	
	if love.keyboard.isDown("left") then
		Player.VX = Player.VX - Player.WalkSpeed
	end
	if love.keyboard.isDown("right") then
		Player.VX = Player.VX + Player.WalkSpeed
	end
	-- Change the player's direction based on velocity
	if Player.VX > 0 then
		Player.Direction = 1
	elseif Player.VX < 0 then
		Player.Direction = 2
	end
	
	
	ChunkBgTop = DecodeHex(PlayerChunk.BgTop)
	ChunkBgBottom = DecodeHex(PlayerChunk.BgBottom)
	BgTop = {
		Lerp(BgTop[1],ChunkBgTop[1],0.05),
		Lerp(BgTop[2],ChunkBgTop[2],0.05),
		Lerp(BgTop[3],ChunkBgTop[3],0.05)
	}
	BgBottom = {
		Lerp(BgBottom[1],ChunkBgBottom[1],0.05),
		Lerp(BgBottom[2],ChunkBgBottom[2],0.05),
		Lerp(BgBottom[3],ChunkBgBottom[3],0.05)
	}
	Radio.TargetMusic = PlayerChunk.Music
	Radio.Update()
	
	CamX = (Player.X*32)-(WindowWidth/2)
	CamY = (Player.Y*32)-(WindowHeight/2)
	
	Animated.UpdateTick()
	
	PhysicsTick()
end

function LoadChunk(X,Y)
	if Chunks[X][Y].Loaded == "true" then
		return Chunks[X][Y]
	end
	local ChunkFileName = X.."_"..Y
	local ChunkData = PlaceholderChunk
	local SaveLocation = "Saves/"..SaveSlot.."/Chunks/"..ChunkFileName
	if love.filesystem.getInfo(SaveLocation) then
		ChunkData = ChunkCompression.DecompressChunk(
			love.filesystem.read(SaveLocation)
		)
	end
	--[[
	local DefaultTry = ChunkCompression.ChunkFromDefaultWorld(ChunkFileName)
	if DefaultTry then
		ChunkData = DefaultTry
		Chunks[X][Y] = DefaultTry
		Chunks[X][Y].Loaded = "true"
		return Chunks[X][Y]
	end--]]
	local DefaultPath = "DefaultWorld/Chunks/"..ChunkFileName
	local CanOpenDefault = pcall(function(Path)assert(love.filesystem.getInfo(Path))end,DefaultPath)
	if CanOpenDefault then
		local Body = love.filesystem.read(DefaultPath)
		local Decompressed = ChunkCompression.DecompressChunk(Body)
		Chunks[X][Y] = Decompressed
		Chunks[X][Y].Loaded = "true"
		return Decompressed
	else
		
	end
	Chunks[X][Y] = ChunkData
	Chunks[X][Y].Loaded = "true"
	return ChunkData
end

function FlipXImageData(ID)
	local Width = ID:getWidth()
	local Height = ID:getHeight()
	local Final = love.image.newImageData(ID,Width,Height)
	Final:mapPixel(0,0,Width,Height,function(X,Y,R,G,B,A)
		local R,G,B,A = ID:getPixel(Width-X,Y)
		return R,G,B,A
	end)
	return Final
end

function love.draw()
	RenderBg()
	love.graphics.setColor(1,1,1)
	
	local PlayerChunk = Chunks[PlayerChunkX][PlayerChunkY]
	local ChunkCamX = (PlayerTileX*32)-(WindowWidth/2)
	local ChunkCamY = (PlayerTileY*32)-(WindowHeight/2)
	for X=-1,1,1 do
		for Y=-1,1,1 do
			RenderChunk = Chunks[PlayerChunkX+X][PlayerChunkY+Y]
			for Z=0,1,1 do
				RenderChunkLayer((1024*X)-ChunkCamX,(1024*Y)-ChunkCamY,RenderChunk,Z)
			end
		end
	end
	
	-- Placeholder mario
	love.graphics.setColor(1,0,0)
	love.graphics.rectangle("fill",WindowWidth/2,WindowHeight/2,Player.W*32,Player.H*32)
	love.graphics.setColor(1,1,1)
	
	-- WERE DRAWING MARIO NOW GUYS OMG EPIC
	local MarioFrame = 1
	---[[
	love.draw(
		MarioSprites[1][1][1],--MarioSprites[Player.Direction][MarioFrame][Player.Powerup+1],
		WindowWidth/2-16,
		WindowHeight/2-48,
		0,
		1,
		1
	)--]]
	
	
	for X=-1,1,1 do
		for Y=-1,1,1 do
			RenderChunk = Chunks[PlayerChunkX+X][PlayerChunkY+Y]
			RenderChunkLayer((1024*X)-ChunkCamX,(1024*Y)-ChunkCamY,RenderChunk,2)
		end
	end
end

function love.keypressed(Key)
	if Key == "z" then
		if Player.Grounded then
			Player.Grounded = false
			Player.VY = -0.75 - math.abs(Player.VX*0.1)
		end
		if Player.WallJumpAllowed then
			if Player.WallJumpDir == 1 then
				Player.VY = -0.35
				Player.VX = 1
			elseif Player.WallJumpDir == 2 then
				Player.VY = -0.35
				Player.VX = -1
			end
		end
	end
end
